
/*
  Software serial multple serial test

 Receives from the hardware serial, sends to software serial.
 Receives from software serial, sends to hardware serial.

 The circuit:
 * RX is digital pin 2 (connect to TX of other device)
 * TX is digital pin 3 (connect to RX of other device)

 Note:
 Not all pins on the Mega and Mega 2560 support change interrupts,
 so only the following can be used for RX:
 10, 11, 12, 13, 50, 51, 52, 53, 62, 63, 64, 65, 66, 67, 68, 69

 Not all pins on the Leonardo support change interrupts,
 so only the following can be used for RX:
 8, 9, 10, 11, 14 (MISO), 15 (SCK), 16 (MOSI).

 created back in the mists of time
 modified 25 May 2012
 by Tom Igoe
 based on Mikal Hart's example

 This example code is in the public domain.

 */
#include <SoftwareSerial.h>

#include <Encoder.h>
#include <Dagu4Motor.h>

//-------------------- RF Antenna Receiver ---------------
SoftwareSerial mySerial(3, 2); // RX, TX

struct Message {
  bool play_sound: 1;
  bool is_X: 1;
  uint8_t value: 6;
};
const uint8_t NEUTRAL_SPEED_VALUE = 31;

//-------------------- Motor Controller ------------------
const int FRONT_RIGHT_PWM = 5; // CH 1
const int FRONT_RIGHT_DIR = 7; // CH 1
const int FRONT_LEFT_PWM = 6; // CH 2
const int FRONT_LEFT_DIR = 8; // CH 2
const int BACK_RIGHT_PWM = 9; // CH 3
const int BACK_RIGHT_DIR = 4; // CH 3
const int BACK_LEFT_PWM = 10; // CH 4
const int BACK_LEFT_DIR = 12; // CH 4
Dagu4Motor FrontLeft(FRONT_LEFT_PWM, FRONT_LEFT_DIR, NULL, NULL, NULL);
Dagu4Motor FrontRight(FRONT_RIGHT_PWM, FRONT_RIGHT_DIR, NULL, NULL, NULL);
Dagu4Motor BackLeft(BACK_LEFT_PWM, BACK_LEFT_DIR, NULL, NULL, NULL);
Dagu4Motor BackRight(BACK_RIGHT_PWM, BACK_RIGHT_DIR, NULL, NULL, NULL);

uint8_t leftSpeed = 0;
uint8_t rightSpeed = 0;
bool dir = true;
bool ldir = true;
void updateMotor(uint8_t leftSpeed, uint8_t rightSpeed, bool dir) {
  FrontLeft.setSpeed(leftSpeed);
  FrontRight.setSpeed(rightSpeed);
  BackLeft.setSpeed(leftSpeed);
  BackRight.setSpeed(rightSpeed);
  FrontLeft.setMotorDirection(dir);
  FrontRight.setMotorDirection(!dir);
  BackLeft.setMotorDirection(!dir);
  BackRight.setMotorDirection(dir);
  
}

void setup()
{
  // Open serial communications and wait for port to open:
  Serial.begin(57600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Native USB only
  }

  // set the data rate for the SoftwareSerial port
  mySerial.begin(57600);

  // Setup motor controller
  FrontLeft.begin();
  FrontLeft.setSpeed(0);
  FrontRight.begin();
  FrontRight.setSpeed(0);
  BackLeft.begin();
  BackLeft.setSpeed(0);
  BackRight.begin();
  BackRight.setSpeed(0);
}

void loop(){ // run over and over 
    
  if (mySerial.available()) {
    byte byteMsg = mySerial.read();
    Message msg = *(Message*) &byteMsg;
    Serial.println();
    if (msg.is_X) {
      if (msg.value > NEUTRAL_SPEED_VALUE) {
        uint8_t value = map(msg.value, 32, 63, 0, 250);
        leftSpeed = value;
        rightSpeed = value;
        dir = true;
        
      } else {
        uint8_t value = map(msg.value, 0, 31, 250, 0);
        leftSpeed = value;
        rightSpeed = value;
        dir = false;
      }
    } else {
      if (msg.value > NEUTRAL_SPEED_VALUE) {
        uint8_t value = map(msg.value, 32, 63, 0, rightSpeed);
        rightSpeed -= value;
      } else {
        uint8_t value = map(msg.value, 0, 31, leftSpeed, 0);
        leftSpeed -= value;
      }
      Serial.println(msg.value);
      Serial.println(leftSpeed);
      Serial.println(rightSpeed);
      Serial.println(dir);
      updateMotor(leftSpeed, rightSpeed, dir);
    }
  }
}
